import random
import re


class ReLetter:
    def __init__(self, regul):
        self.regul = regul

    def __repr__(self):
        return f"ReLetter class({self.regul})"

    def __str__(self):
        return f"ReLetter class({self.regul})"

    def reg_string_generator(self, char_lib, not_space=False):  # Получает строку формата [.+]{\d+} или [.+]
        chars = ""
        reg = self.regul
        if "[^" in reg:
            chars = re.findall(reg, char_lib)
            if not_space:
                chars = "".join(chars).replace(" ", "")
                assert len(chars) > 0, "Can't form string witch will be splitted"
            return random.choice(chars)
        reg = reg[1:-1]
        if "\\" in reg:
            spes = re.findall(r"\\.", reg)
            for let in spes:
                chars += let[1]
                reg = reg.replace(let, "")
        if "-" in reg:
            spes = re.findall(r".-.", reg)
            for let in spes:
                chars += "".join(chr(i + ord(let[0])) for i in range(1 + ord(let[2]) - ord(let[0])))
                reg = reg.replace(let, "")
        chars += reg
        if not_space:
            chars = "".join(chars).replace(" ", "")
            assert len(chars) > 0, "Can't form string witch will be splitted"
        return random.choice(chars)


class ReMultiplier:
    def __init__(self, regul):
        if "-" in regul:
            mult_range = range(int(regul.split(",")[0][1:]), int(regul.split(",")[1][:-1]))
            regul = random.choice(mult_range)
        else:
            regul = int(regul[1:-1])
        self.regul = regul

    def __repr__(self):
        return f"ReDiv class({self.regul})"

    def __str__(self):
        return f"ReDiv class({self.regul})"



from re_generate.classes import ReLetter, ReMultiplier


def brackets_divider(in_str, container, left_br, right_br):
    temp = ""
    export_li = []
    for i, val in enumerate(in_str):
        if (val == left_br and i == 0) or (val == left_br and temp == "" and in_str[i - 1] != "\\"):
            temp += val
        elif temp != "" and val == right_br and in_str[i - 1] != "\\":
            temp += val
            export_li.append(container(temp))
            temp = ""
        elif temp != "":
            temp += val
        else:
            if len(export_li) == 0 or not isinstance(export_li[-1], str):
                export_li.append(val)
            else:
                export_li[-1] += val
    return export_li


def re_letters_former(reg_str: str) -> list:
    return brackets_divider(reg_str, ReLetter, "[", "]")


def re_multiplier_former(reg_list: list) -> list:
    temp = []
    for cont in reg_list:
        if isinstance(cont, str):
            for val in brackets_divider(cont, ReMultiplier, "{", "}"):
                temp.append(val)
        else:
            temp.append(cont)
    return temp


def variant_spliter(reg_list: list) -> list:
    for i, container in enumerate(reg_list):
        if isinstance(container, str):
            for j, value in enumerate(container):
                if (value == "|" and j == 0) or (value == "|" and container[j - 1] != "\\"):
                    left = (reg_list[:i] + [container[:j]]) if container[:j] != "" else reg_list[:i]
                    right = ([container[j + 1:]] + reg_list[i + 1:]) if container[j + 1:] != "" else reg_list[i + 1:]
                    return [left, *variant_spliter(right)]
    return [reg_list]


def string_divider(reg_list: list) -> list:
    temp = []
    for container in reg_list:
        if isinstance(container, str):
            for i, value in enumerate(container):
                if value == "\\" and container[i + 1] == "\\":
                    temp.append(value)
                elif value != "\\":
                    temp.append(value)
        else:
            temp.append(container)
    return temp


def re_letters_multiplier(reg_list: list) -> list:
    temp = []
    for i, container in enumerate(reg_list):
        if isinstance(container, ReMultiplier):
            for _ in range(container.regul):
                temp.append(reg_list[i - 1])
        elif i + 1 == len(reg_list) or not isinstance(reg_list[i + 1], ReMultiplier):
            temp.append(container)
    return temp


def re_letters_activator(reg_list: list, char_lib, will_be_stripped) -> list:
    temp = []
    if will_be_stripped:
        for i, container in enumerate(reg_list):
            if isinstance(container, ReLetter):
                if i == 0:
                    temp.append(container.reg_string_generator(True))
                else:
                    temp.append(container.reg_string_generator(char_lib))
            else:
                assert i != 0 or container != " ", "Can't form string witch will be splitted"
                temp.append(container)
    else:
        for container in reg_list:
            if isinstance(container, ReLetter):
                temp.append(container.reg_string_generator(char_lib))
            else:
                temp.append(container)
    return temp



import random
from re_generate.cfg_constants import *

from re_generate.handlers import re_letters_former, re_multiplier_former, variant_spliter, string_divider, \
    re_letters_multiplier, re_letters_activator


class ReGenerator:

    def __init__(self):
        self.will_be_stripped = WILL_BE_STRIPPED
        self.char_lib = CHAR_LIB

    def re_generator(self, regular_expression: str) -> str:
        temp = re_letters_former(regular_expression)
        temp = re_multiplier_former(temp)
        temp = random.choice(variant_spliter(temp))
        temp = string_divider(temp)
        temp = re_letters_multiplier(temp)
        temp = re_letters_activator(temp, self.char_lib, self.will_be_stripped)
        assert len(temp) > 0, "Wrong ReGex! can't generate anything from it!"
        return "".join(temp)

    def re_generator_list(self, regular_expression: str, amount: int) -> list:
        return [self.re_generator(regular_expression) for _ in range(amount)]



from setuptools import setup, find_packages
import pathlib

here = pathlib.Path(__file__).parent.resolve()
long_description = (here / "README.md").read_text(encoding="utf-8")


setup(
    name="re-generate",
    version="0.1.3",
    description="Package for random data generation via ReGex",
    long_description=long_description,
    url="https://github.com/Warrfie/re-generate",
    author="Kuklikov Maxim (Warrfie)",
    author_email="warrfie@gmail.com",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Information Technology",
        "Topic :: Software Development :: Quality Assurance",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
    ],
    keywords="QA, random string, reverse regular expression, regex generator, testing, autotesting",
    packages=find_packages(),
    python_requires=">=3.7, <4",
    project_urls={
        "Telegram": "https://t.me/sasisochka",
        "Main page": "https://github.com/Warrfie/re-generate"
    },
)
